*
      program compile_results
*
* compile_results: Read output files from runs with some number
*                  of charges. Essentially the inverse of make_runs.
*
* Update history:
*    v1.0  21-May-2004  Original version
*    v1.1  26-May-2004  Fix bug
*    v1.2  28-May-2004  Add make_file
*
      implicit none
*
* Include file
*
      include 'eval.inc'
*
* Parameters
*    CNAME  = c*15 = code name
*    CUPDTE = c*11 = code date
*    CVERS  = c*4  = code version
*
      character*11 CUPDTE
      character*15 CNAME
      character*4  CVERS
      parameter   (CNAME='compile_results')
      parameter   (CVERS='v1.2')
      parameter   (CUPDTE='28-May-2004')
*
* Subprogrammes
*    econf     = r*4 function = energy of configuration per charge
*    iargc     = i*4 function = number of arguments
*    make_file = subroutine = make file name
*
      real*4    econf,ranuni
      integer*4 iargc
      external  econf,ranuni,strhed,strlen
*
* Local variables
*    ang1   = r*4        = angle-1 (theta; radians)
*    ang2   = r*4        = angle-2 (phi; radians)
*    buff   = c*132      = input buffer
*    cfile  = c*132      = input file name
*    croot  = c*132      = input file root
*    csave  = c*132      = name of file with best solution
*    ctitle = c*132      = title string
*    dtr    = r*8        = degrees/radians conversion factor
*    emin   = r*4        = minimum found (single run)
*    esave  = r*4        = minimum found (all runs)
*    i0     = i*4        = counting variable
*    i1     = i*4        = counting variable
*    i2     = i*4        = counting variable
*    ia     = i*4        = counting variable over particles a
*    ib     = i*4        = counting variable over particles b
*    ierror = i*4        = error code
*    iloop  = i*4        = counter over input files
*    n0     = i*4        = counter over neighbours
*    nc     = i*4        = number of particles
*    nfile  = i*4        = length of file name
*    nloop  = i*4        = number of input files
*    nroot  = i*4        = length of file root string
*    ntitle = i*4        = length of title string
*    nvec   = i*4        = length of vectors in minimization
*    rab    = r*4        = separation of particles a,b
*    rmin   = r*4        = minimum separation of particles b from a
*    s1     = r*8        = accumulator for sum
*    s2     = r*8        = accumulator for sum
*    x1     = r*4        = input value
*    x2     = r*4        = input value
*    x3     = r*4        = input value
*    x4     = r*4        = input value
*    x5     = r*4        = input value
*    x6     = r*4        = input value
*    xa     = r*4        = x-location of particle a
*    xab    = r*4        = x-separation of a and b
*    xb     = r*4        = x-location of particle b
*    ya     = r*4        = y-location of particle a
*    yab    = r*4        = y-separation of a and b
*    yb     = r*4        = y-location of particle b
*    za     = r*4        = z-location of particle a
*    zab    = r*4        = z-separation of a and b
*    zb     = r*4        = z-location of particle b
*    xsave  = r*4(NXMAX) = best vector found
*    xvec   = r*4(NXMAX) = parameter vector in minimization
*    xvhi   = r*4(NXMAX) = parameter vector high bound
*    xvlo   = r*4(NXMAX) = parameter vector low bound
*
      character*132 buff,cfile,croot,csave,ctitle
      real*8        dtr,s1,s2
      real*4        ang1,ang2,emin,esave,rab,rmin,x1,x2,x3,x4,
     +              x5,x6,xa,xab,xb,ya,yab,yb,za,zab,zb
      real*4        xsave(NXMAX),xvec(NXMAX),xvhi(NXMAX),
     +              xvlo(NXMAX)
      integer*4     i0,i1,i2,ia,ib,ierror,iloop,n0,
     +              nc,nfile,nloop,nroot,ntitle,nvec
*
      dtr=atan(1.0d0)/45.0d0
*
* Compose and write title
*
      call strhed(CNAME,CVERS,CUPDTE,ctitle)
      call strlen(ctitle,ntitle)
      write (6,600) ctitle(1:ntitle)
*
* Get number of charges and files and input file name root
*
      if (iargc() .eq. 3) then
*
        call getarg(1,buff)
        read (buff,*) nc
        write (6,601) nc
*
        call getarg(2,buff)
        read (buff,*) nloop
        write (6,602) nloop
*
        call getarg(3,croot)
        call strlen(croot,nroot)
        write (6,603) croot(1:nroot)
*
      else
*
* Print usage message and stop
*
        write (6,697)
        stop ' '
*
      endif
*
* Get number of parameters. Since I did not force particle 1 to
* be on the z-axis explicitly (it is done implicitly using
* the bounds array), this is twice the number of particles
*
      nvec=2*nc
*
* Check not too many particles
*
      if     (nvec .gt. NXMAX) then
        write (6,699) 
        stop ' '
      elseif (nvec .lt.     4) then
        write (6,698)
        stop ' '
      endif
*
* Loop to read required number of output files
*
      do iloop=1,nloop
*
* Make input file name
*
        call make_file(cfile,nfile,croot,nc,iloop,'.out')
*
* Read results from input file
*
        open(unit=4,name=cfile(1:nfile),status='old',
     +       iostat=ierror)
        if (ierror .ne. 0) then
          write (6,696)
          stop ' '
        endif
*
        dowhile (ierror .eq. 0)
          read (4,400,iostat=ierror) buff
          if (ierror .eq. 0) then
            if (buff(1:1) .ne. '#') then
*
* Not a comment line: read charge index, angle data
*
              read (buff,*) i0,x1,x2,x3,x4,x5,x6
*
* Store data
*
              i1      =2*i0-1
              xvlo(i1)=x1
              xvec(i1)=x2
              xvhi(i1)=x3
              i2      =2*i0
              xvlo(i2)=x4
              xvec(i2)=x5
              xvhi(i2)=x6
*
            endif
          endif
        enddo
*
* Close file
*
        close (unit=4)
*
* Check i0 vs nc
*
        if (i0 .ne. nc) then
          write (6,695)
        endif
*
* Get current energy
*
        emin=econf(xvec,nvec)
*
* Calculate mean and s.d. of separations between nearest neighbours.
* For each charge, calculate distances to each other charge and take
* minimum distance.
*
        n0=0
        s1=0.0d0
        s2=0.0d0
*
        do ia=1,nc
*
          i1=2*ia-1
          i2=i1+1
*
          ang1=dtr*xvec(i1)
          ang2=dtr*xvec(i2)
*
          xa=sin(ang1)*cos(ang2)
          ya=sin(ang1)*sin(ang2)
          za=cos(ang1)
*
* Set initial value of rmin to diameter of sphere (squared)
*
          rmin=4.0
*
          do ib=1,nc
*
* Only consider other particles
*
            if (ia .ne. ib) then
*
              i1=2*ib-1
              i2=i1+1
*
              ang1=dtr*xvec(i1)
              ang2=dtr*xvec(i2)
*
              xb=sin(ang1)*cos(ang2)
              yb=sin(ang1)*sin(ang2)
              zb=cos(ang1)
* 
              xab=xa-xb
              yab=ya-yb
              zab=za-zb
              rab=xab*xab+yab*yab+zab*zab
              rmin=min(rmin,rab)
*
            endif
*
          enddo
*
          n0=n0+1
*
          if (rmin .gt. 0.0) then
            s1=s1+sqrt(rmin)
            s2=s2+rmin
          endif
*
        enddo
*
* Write brief report
*
        if (n0 .ge. 1) then
          xab=s1/float(n0)
          s2 =s2-s1*xab
          if     (s2 .le. 0.0d0) then
            yab=0.0
          elseif (n0 .ge.     2) then
            yab=sqrt(s2/float(n0-1))
          else
            yab=0.0
          endif
          if (iloop .eq. 1) then
            write (6,604)
          endif
          i1=max(nfile,32)
          write (6,605) iloop,cfile(1:i1),emin,xab,yab
        endif
*
* Look for minimum of all runs, and save if so
*
        if (iloop .eq. 1) then
*
          do i1=1,nc
            xsave(i1)=xvec(i1)
          enddo
          csave=cfile
          esave=emin
*
        elseif (emin .lt. esave) then
*
          do i1=1,nc
            xsave(i1)=xvec(i1)
          enddo
          csave=cfile
          esave=emin
*
        endif
*
* End of file reading loop
*
      enddo
*
* Print best result
*
      call strlen(csave,nfile)
      write (6,606) csave(1:nfile),esave
      do ia=1,nc
        i1=2*ia-1
        i2=i1+1
        xa=sin(dtr*xvec(i1))*cos(dtr*xvec(i2))
        ya=sin(dtr*xvec(i1))*sin(dtr*xvec(i2))
        za=cos(dtr*xvec(i1))
        write (6,607) ia,xa,ya,za
      enddo
*
* End of run
*
      write (6,608) ctitle(1:ntitle)
*
 100  format (a,'_',i2.2,'_',i6.6,'.out')
*
 400  format (a)
*
 600  format (' '/
     +        ' ',a//
     +        '    Evaluate output files'/)
 601  format ('     Number of charges = ',i4)
 602  format ('       Number of files = ',i4)
 603  format ('        File root name = ',a)
 604  format (' '/
     +        ' loop',2x,'file',t46,'energy',7x,'NN separation')
 605  format (' ',i4,2x,a,t40,1pe15.5,2x,0pf6.4,' +/- ',f6.4)
 606  format (' '/
     +        '      File with best solution: ',a/
     +        '                       Energy: ',1pe15.5//
     +        '      particle',8x,'x',8x,'y',8x,'z')
 607  format (' ',5x,i8,3f9.4)
 608  format (' '/
     +        ' ',a)
*
 695  format (' *** Error: number of charges mismatch ***')
 696  format (' *** Error: input file access error ***')
 697  format (' '/
     +        ' Usage: compile_results nc nl inroot'/)
 698  format (' *** Error: too few particles for interest ***')
 699  format (' *** Error: too many particles for arrays  ***')
*
      end
