*
      real*4 function econf(an,na)
*
* econf: Calculate the energy per charge from a set of
*        charges with locations specified by AN(1,2,..NA)
*        at fixed distance from the origin. 
*
* Update history:
*    v1.0   8-Oct-1992  Original version
*    v2.0  23-Oct-1992  Internal calculations r*8
*    v3.0  15-May-2004  Linux retread
*    v3.1  26-May-2004  Use eval.inc
*
      implicit none
*
* Arguments:
*    an = t*4(?) = vector of angles
*    na = i*4    = number of angles in vector
*
      real*4    an(1)
      integer*4 na
*
* Include file
*
      include 'eval.inc'
*
* Local variables:
*   esum = r*8 = summed energy
*   i1   = i*4 = counter
*   i2   = i*4 = counter
*   ia   = i*4 = counter over pair particles
*   ib   = i*4 = counter over pair particles
*   nn   = i*4 = number of particles
*   xa   = r*8 = location of particle a
*   ya   = r*8 = location of particle a
*   za   = r*8 = location of particle a
*   xb   = r*8 = location of particle b
*   yb   = r*8 = location of particle b
*   zb   = r*8 = location of particle b
*   rab  = r*8 = separation of a and b
*   xab  = r*8 = separation of a and b
*   yab  = r*8 = separation of a and b
*   zab  = r*8 = separation of a and b
*
      integer*4 i1,i2,ia,ib,nn
      real*8    dtr,esum,pi,rab,xa,xab,xb,ya,yab,yb,za,zab,zb
*
* Initialise
*
      pi  =4.0d0*atan(1.0d0)
      dtr =pi/180.0d0
      nn  =na/2
      esum=0.0d0
*
* For each pair of charges, sum their mutual energy in units of
*    q^2/4 pi epsilon_0
*
      do ia=1,nn
*
* Location of first particle in pair (Cartesian coordinates)
*
        i1=2*ia-1
        i2=i1+1
        xa=sin(dtr*an(i1))*cos(dtr*an(i2))
        ya=sin(dtr*an(i1))*sin(dtr*an(i2))
        za=cos(dtr*an(i1))
*
        do ib=(ia+1),nn
*
* Location of second particle in pair (Cartesian coordinates)
*
          i1=2*ib-1
          i2=i1+1
          xb=sin(dtr*an(i1))*cos(dtr*an(i2))
          yb=sin(dtr*an(i1))*sin(dtr*an(i2))
          zb=cos(dtr*an(i1))
* 
* Particle separations
*
          xab=xa-xb
          yab=ya-yb
          zab=za-zb
          rab=sqrt(xab*xab+yab*yab+zab*zab)
*
* Guard against zero separations
*
          if (rab .lt. rabmin) then
            esum=esum+EABMAX
          else
            esum=esum+(1.0d0/rab)
          endif
*
        enddo
*
      enddo
*
* Normalize by number of particles
*
      econf=esum/float(nn)
*
* End
*
      end
