c
      subroutine error(func,delt,xvlo,xelo,xvec,xehi,xvhi,nvec)
c
c error: Take best-fit and get error bounds (still pretty crude)
c
c Update history
c    V1.0   1-Nov-1993  Original version
c    V1.1  12-Nov-1993  Improve algorithm slightly
c    V1.2  10-Jul-1998  Small improvements
c    V1.3  20-Jul-1998  Add check for new minimum
c
      implicit none
c
c Arguments
c    func = r*4 function = quantity to minimize
c    delt = r*4          = change in function corresponding to errors
c    xvlo = r*4(nvec)    = array of lower limits to xvec
c    xelo = r*4(nvec)    = array of error lower bounds to xvec
c    xvec = r*4(nvec)    = array of quantities to find at minimum
c    xehi = r*4(nvec)    = array of error upper bounds to xvec
c    xvhi = r*4(nvec)    = array of upper limits to xvec
c    nvec = i*4          = number of parameters
c
      integer*4 nvec
      real*4    func
      external  func
      real*4    delt,xvlo(nvec),xelo(nvec),xvec(nvec),xehi(nvec),
     +          xvhi(nvec)
c
c Subprogrammes
c    getop = subroutine = get controlling parameter
c
      external getop
c
c Local parameter
c    NVMAX = i*4 = maximum size of vectors that can be dealt with
c
      integer*4  NVMAX
      parameter (NVMAX=128)
c
c Local variables
c    eps     = r*4        = convergence criterion
c    fmin    = r*4        = minimum of function
c    fval    = r*4        = current function value
c    i1      = i*4        = counting index
c    iprint  = i*4        = print level
c    nimax   = i*4        = maximum number of tests
c    ntest   = i*4        = number of error-seeking tests
c    test    = r*4        = test variable
c    xtst(i) = r*4(NVMAX) = current test vector
c    value   = r*4        = value of parameter
c    xa      = r*4        = parameter value bracketing bound
c    xb      = r*4        = parameter value bracketing bound
c
      real*4    eps,fmin,fval,test,value,xa,xb
      real*4    xtst(NVMAX)
      integer*4 i1,iprint,nimax,ntest
c
c Get value of controlling parameters
c
      call getop('ERROR.IPRINT',value)
      iprint=ifix(value+0.5)
      call getop('ERROR.NIMAX' ,value)
      nimax =ifix(value+0.5)
      call getop('ERROR.EPS'   ,value)
      eps   =value
c
c Title
c
      if (iprint .gt. 0) then
        write (6,600)
      endif
c
c Check that vector length isn't too large
c
      if (nvec .gt. NVMAX) then
        write (6,699)
        return
      endif
c
c Get best fit
c
      fmin=func(xvec,nvec)
c
c Report convergence criteria
c
      if (iprint .gt. 1) then
        write (6,601) nimax,eps
      endif
c
c Initialise errors to overall bounds, initialise test vector
c
      do i1=1,nvec
        xelo(i1)=xvlo(i1)
        xehi(i1)=xvhi(i1)
        xtst(i1)=xvec(i1)
      enddo
c
c Now, for each variable, interval halve in vector value until
c    get within accuracy eps of the error-edge. ASSUMES MONOTONIC
c    in this direction. Range of variables = zero is nicely dealt
c    with by the test method.
c
      do i1=1,nvec
c
c Lower limit
c
        xa   =xvlo(i1)
        xb   =xvec(i1)
        test =100.0*eps
        ntest=0
        dowhile ((abs(test) .gt. eps) .and. (ntest .lt. nimax))
          xtst(i1)=0.5*(xa+xb)
          fval    =func(xtst,nvec)
c
          test    =fval-(fmin+delt)
          ntest   =ntest+1
          if (iprint .gt. 1) then
            write (6,602) xa,xtst(i1),xb,fmin,fval,test
          endif
c
c If fval < fmin, then the minimum found before wasn't good
c    enough. Report problem.
c
          if ((fval .lt. fmin) .and. (iprint .gt. 0)) then
            write (6,603) i1,fval,fmin
          endif
c
          if     (test .gt. 0.0) then
            xa=xtst(i1)
          elseif (test .lt. 0.0) then
            xb=xtst(i1)
          endif
c
        enddo
        xelo(i1)=0.5*(xa+xb)
c
c Upper limit
c
        xa   =xvhi(i1)
        xb   =xvec(i1)
        test =100.0*eps
        ntest=0
        dowhile ((abs(test) .gt. eps) .and. (ntest .lt. nimax))
          xtst(i1)=0.5*(xa+xb)
          fval    =func(xtst,nvec)
c
          test    =fval-(fmin+delt)
          ntest   =ntest+1
          if (iprint .gt. 1) then
            write (6,602) xb,xtst(i1),xa,fmin,fval,test
          endif
c
c If fval < fmin, then the minimum found before wasn't good
c    enough. Report problem.
c
          if ((fval .lt. fmin) .and. (iprint .gt. 0)) then
            write (6,603) i1,fval,fmin
          endif
c
          if     (test .gt. 0.0) then
            xa=xtst(i1)
          elseif (test .lt. 0.0) then
            xb=xtst(i1)
          endif
c
        enddo
        xehi(i1)=0.5*(xa+xb)
c
c Reset xtst(i1)
c
        xtst(i1)=xvec(i1)
c
      enddo
c
c End
c
      if (iprint .gt. 0) then
        write (6,604)
      endif
c
 600  format (' error: v1.3')
 601  format (' error:  maximum number of tests = ',i5/
     +        '               accuracy required = ',1pe15.5)
 602  format (' error: lower x, test x, upper x = ',3(1pe15.5)/
     +        '               minimum f, test f = ',2(1pe15.5)/
     +        '                  test parameter = ',  1pe15.5 )
 603  format (' error: lower minimum found, par = ',i5/
     +        '               test f, minimum f = ',2(1pe15.5))
 604  format (' ')
c
 699  format (' *** Error: error: parameter vector too large ***')
c
      end
