*
      subroutine lnmnop(f,plo,p,phi,xi,n,fret)
*
* lnmnop: Find linear minimum of function f
*
* Update history
*    v1.0  18-Nov-1992  Original (from Press et al LINMIN)
*    v2.0  16-Jun-1994  Change name and tidy up
*    v2.1  20-Jul-1998  Small improvements
*    v2.2   4-Aug-1999  Change comment style
*
      implicit none
*
* Arguments
*    f      = r*4 function = function to optimize
*    plo(i) = r*4(n)       = lower limit vector
*    p(i)   = r*4(n)       = vector of parameters
*    phi(i) = r*4(n)       = upper limit vector
*    xi(i)  = r*4(n)       = shift vector of parameters
*    n      = i*4          = number of parameters
*    fret   = r*4          = value at minimum
*
      real*4    f
      external  f
      integer*4 n
      real*4    fret,plo(n),p(n),phi(n),xi(n)
*
* Local parameters
*    TINY = r*4 = limit for equality of r*4 numbers
*    TOL  = r*4 = accuracy demanded in brntop
*
      real*4     TINY,TOL
      parameter (TINY=1.0e-12)
      parameter (TOL =1.0e-06)
*
* Subprogrammes
*    mnbrop = subroutine   = bracketing routine
*    brntop = r*4 function = locate bracketed minimum
*
      real*4   brntop
      external brntop,mnbrop
*
* Local variables
*    ax   = r*4 = location of point
*    xx   = r*4 = location of point
*    bx   = r*4 = location of point
*    fa   = r*4 = value of function
*    fx   = r*4 = value of function
*    fb   = r*4 = value of function
*    i1   = i*4 = counting index
*    xmin = r*4 = minimum of shift scalar
*
      integer*4 i1
      real*4    ax,xx,bx,fa,fx,fb,xmin
*
* Find points ax,xx,bx bracketing minimum. 
*
      ax=0.0
      xx=1.0
      bx=2.0
      call mnbrop(ax,xx,bx,fa,fx,fb,f,plo,p,phi,xi,n)
*
* If the value of the function is the same at ax, xx, and bx, then
* we have a null direction and minimization in this direction is
* pointless. Set the amount of motion in this direction to zero
* and return. Otherwise, perform minimization in this direction.
*
      if ((abs(fa-fx) .gt. TINY) .or.
     +    (abs(fx-fb) .gt. TINY)       ) then
*
* Perform minimization in 1D using Brent's method to fractional
* accuracy TOL in f(x)
*
        fret=brntop(ax,xx,bx,f,plo,p,phi,xi,n,TOL,xmin)
*
      else
        fret=fx
        xmin=xx
      endif
*
* Shift input vector to result position
*
      do i1=1,n
	xi(i1)=xmin*xi(i1)
	p(i1) =max(plo(i1),min(p(i1)+xi(i1),phi(i1)))
      enddo
*
* End
*
      end
