*
      program make_runs
*
* make_runs: Make input files for runs with some number of charges
*            ready for passing to eval. Use a random number sequence.
*
* Update history:
*    v1.0  21-May-2004  Original version
*    v1.1  25-May-2004  Smaller search patches
*    v1.2  28-May-2004  Add make_file routine
*
      implicit none
*
* Include file
*
      include 'eval.inc'
*
* Subprogrammes
*    econf     = r*4 function = energy of configuration per charge
*    iargc     = i*4 function = number of arguments
*    make_file = subroutine   = create file name
*
      real*4    econf,ranuni
      integer*4 iargc
      external  econf,ranuni,make_file,strhed,strlen
*
* Parameters
*    CNAME  = c*9  = code name
*    CUPDTE = c*11 = code date
*    CVERS  = c*4  = code version
*
      character*11 CUPDTE
      character*9  CNAME
      character*4  CVERS
      parameter   (CNAME='make_runs')
      parameter   (CVERS='v1.2')
      parameter   (CUPDTE='28-May-2004')
*
* Local variables
*    buff   = c*132       = input buffer
*    cfile  = c*132       = output file name
*    croot  = c*132       = output file root
*    ctitle = c*132       = title string
*    emin   = r*4         = minimum found
*    i0     = i*4         = counting variable
*    i1     = i*4         = counting variable
*    i2     = i*4         = counting variable
*    i3     = i*4         = counting variable
*    nc     = i*4         = number of particles
*    ntitle = i*4         = length of title string
*    nvec   = i*4         = length of vectors in minimization
*    xvec   = r*4(NXMAX)  = parameter vector in minimization
*    xvhi   = r*4(NXMAX)  = parameter vector high bound
*    xvlo   = r*4(NXMAX)  = parameter vector low bound
*
      character*132 buff,cfile,croot,ctitle
      real*4        emin,x1,x2
      real*4        xvec(NXMAX),xvhi(NXMAX),xvlo(NXMAX)
      integer*4     i0,i1,i2,i3,idum,ierror,iloop,
     +              nc,nfile,nloop,nroot,ntitle,nvec
*
* Compose and write title
*
      call strhed(CNAME,CVERS,CUPDTE,ctitle)
      call strlen(ctitle,ntitle)
      write (6,600) ctitle(1:ntitle)
*
* Get number of charges and files and output file name root
*
      if (iargc() .eq. 3) then
        call getarg(1,buff)
        read (buff,*) nc
        write (6,601) nc
*
        call getarg(2,buff)
        read (buff,*) nloop
        write (6,602) nloop
*
        call getarg(3,croot)
        call strlen(croot,nroot)
        write (6,603) croot(1:nroot)
      else
        write (6,697)
        stop ' '
      endif
*
* Initialise seed random number
*
      idum=14391
*
* Get number of parameters. Since I will not force particle 1 to
* be on the z-axis explicitly (it is done implicitly using
* the bounds array), this is twice the number of particles
*
      nvec=2*nc
*
* Check not too many particles
*
      if     (nvec .gt. NXMAX) then
        write (6,699) 
        stop ' '
      elseif (nvec .lt.     4) then
        write (6,698)
        stop ' '
      endif
*
* Calculate patch sizes in both angles as the random size with
* some safety factor.
*
      x1=2.0*180.0/sqrt(float(nloop))
      x2=2.0*360.0/sqrt(float(nloop))
*
* Loop to create required number of starting files
*
      do iloop=1,nloop
*
* Initialize vectors of angles, lower limits, upper limits
* (all in degrees). The starting position is all particles
* except the first lying in the x-y plane. Note that I fix
* particle 1 to lie on the z axis, particle 2 to lie 
* in the x-z plane, and particle 3 to lie on the +y side
* of the x-z plane. The size of the patches is set by the
* number of loops.
*
        do i0=1,nc
*
          i1=2*i0-1
          i2=i1+1
*
          xvec(i1)=180.0*ranuni(idum)
          xvlo(i1)=max(  0.0,(xvec(i1)-x1))
          xvhi(i1)=min(180.0,(xvec(i1)+x1))
*
          xvec(i2)=360.0*ranuni(idum)
          xvlo(i2)=max(  0.0,(xvec(i2)-x2))
          xvhi(i2)=min(360.0,(xvec(i2)+x2))
*
        enddo
*
        xvlo(1)=  0.0
        xvec(1)=  0.0
        xvhi(1)=  0.0
        xvlo(2)=180.0
        xvec(2)=180.0
        xvhi(2)=180.0
*
        xvlo(4)=  0.0
        xvec(4)=  0.0
        xvhi(4)=  0.0
*
        xvlo(6)=  0.0
        xvhi(6)=180.0
*
* Get current energy
*
        emin=econf(xvec,nvec)
*
* Make output file name
*
        call make_file(cfile,nfile,croot,nc,iloop,'.in')
*
* Write results to output
*
        open(unit=7,name=cfile,status='new',iostat=ierror)
        if (ierror .ne. 0) then
          write (6,696)
          stop ' '
        endif
*
        call strlen(cfile,nfile)
        write (7,700) cfile(1:nfile)
        write (7,701) emin
        do i1=1,nc
          i2=2*i1-1
          i3=2*i1
          write (7,702) i1,xvlo(i2),xvec(i2),xvhi(i2),
     +                     xvlo(i3),xvec(i3),xvhi(i3)
        enddo
        write (7,703) nc
*
* Close file
*
        close (unit=7)
*
* Write report
*
        write (6,604) iloop,emin,cfile(1:nfile)
*
* End of generation loop
*
      enddo
*
* End of run
*
      write (6,605) ctitle(1:ntitle)
*
 100  format (a,'_',i2.2,'_',i6.6,'.in')
*
 400  format (a)
 401  format (' ',a)
*
 600  format (' '/
     +        ' ',a//
     +        '    Create run files'/)
 601  format ('     Number of charges = ',i2)
 602  format ('       Number of files = ',i4)
 603  format ('        File root name = ',a)
 604  format (' Loop ',i4,'; energy = ',1pe15.5,'; file = ',a)
 605  format (' '/
     +        ' ',a)
*
 696  format (' *** Error: output file access error ***')
 697  format (' '/
     +        ' Usage: make_runs nc nl outroot'/)
 698  format (' *** Error: too few particles for interest ***')
 699  format (' *** Error: too many particles for arrays  ***')
*
 700  format ('#'/
     +        '# ',a/
     +        '#')
 701  format ('# emin = ',1pe15.5/
     +        '#')
 702  format (i3,6f10.4)
 703  format ('#'/
     +        '#',i3,' charges'/
     +        '#')
*
      end
