c
      subroutine optim(func,xvlo,xvec,xvhi,nvec,iopt)
c
c optim: Master optimization test routine. Uses a number of methods to
c        look for minima of function func as the nvec parameters in xvec
c        are varied between their limits of xvlo and xvhi.
c
c Update history
c    V1.0   8-Oct-1992  Original version
c    V1.1   2-Aug-1993  Slight updates
c    V1.2   1-Nov-1993  Printing according to parameter in common
c    V1.3  17-Jun-1994  Add best fit function printout
c    V1.4  20-Jul-1998  Tidy up
c
      implicit none
c
c Arguments
c    func = r*4 function = function to optimize
c    xvlo = r*4(nvec)    = array of lower limits to xvec
c    xvec = r*4(nvec)    = array of quantities to find at minimum
c    xvhi = r*4(nvec)    = array of upper limits to xvec
c    nvec = i*4          = number of parameters
c    iopt = i*4          = minimization type
c
      integer*4 iopt,nvec
      real*4    func
      external  func
      real*4    xvlo(nvec),xvec(nvec),xvhi(nvec)
c
c Subprogrammes
c    grids = subroutine = perform grid search
c    monte = subroutine = perform Monte-Carlo search
c    powll = subroutine = perform Powell minimization
c    smplx = subroutine = perform simplex minimization
c    siman = subroutine = perform simulated annealing
c    simmu = subroutine = perform simulated annealing, multiple-moves
c    simpo = subroutine = perform SA + Powell
c    simpu = subroutine = perform SA(multiple) + Powell
c    monpo = subroutine = perform Monte-Carlo + Powell search
c    gripo = subroutine = perform Grid + Powell search
c
      external grids,monte,powll,smplx,siman,simmu,simpo,simpu
      external monpo,gripo
c
c Local variables
c    i1     = i*4 = counting variable
c    iprint = i*4 = print level
c    value  = r*4 = value returned by getop
c
      real*4    value
      integer*4 i1,iprint
c
c Get print level
c
      call getop('OPTIM.IPRINT',value)
      iprint=nint(value)
c
c Starting message
c
      if (iprint .gt. 0) then
        write (6,600) iopt
      endif
c
c Write initial vector and value
c
      if (iprint .gt. 1) then
        write (6,601) 'initial values'
        do i1=1,nvec
          write (6,602) xvlo(i1),xvec(i1),xvhi(i1)
        enddo
      endif
c
      if (iprint .gt. 0) then
        write (6,603) func(xvec,nvec)
      endif
c
c Branch according to option
c
      if     (iopt .eq.  1) then
        call grids(func,xvlo,xvec,xvhi,nvec)
      elseif (iopt .eq.  2) then
        call monte(func,xvlo,xvec,xvhi,nvec)
      elseif (iopt .eq.  3) then
        call powll(func,xvlo,xvec,xvhi,nvec)
      elseif (iopt .eq.  4) then
        call smplx(func,xvlo,xvec,xvhi,nvec)
      elseif (iopt .eq.  5) then
        call siman(func,xvlo,xvec,xvhi,nvec)
      elseif (iopt .eq.  6) then
        call simmu(func,xvlo,xvec,xvhi,nvec)
      elseif (iopt .eq.  7) then
        call simpo(func,xvlo,xvec,xvhi,nvec)
      elseif (iopt .eq.  8) then
        call simpu(func,xvlo,xvec,xvhi,nvec)
      elseif (iopt .eq.  9) then
        call monpo(func,xvlo,xvec,xvhi,nvec)
      elseif (iopt .eq. 10) then
        call gripo(func,xvlo,xvec,xvhi,nvec)
      else
        write (6,699)
      endif
c
c Write final vector and value
c
      if (iprint .gt. 1) then
        write (6,601) 'final values'
        do i1=1,nvec
          write (6,602) xvlo(i1),xvec(i1),xvhi(i1)
        enddo
      endif
c
      if (iprint .gt. 0) then
        write (6,603) func(xvec,nvec)
      endif
c
c End
c
 600  format (' optim: v1.4'/
     +        ' optim: option = ',i2)
 601  format (' optim: ',a/
     +        ' optim:',8x,'lower bound',5x,'central vector',
     +                  8x,'upper bound')
 602  format (' optim:',3(1pe19.5))
 603  format (' optim: function = ',1pe19.5)
c
 699  format (' *** Error: optim: illegal option ***')
c
      end
