*
      subroutine putop(cvar,value)
*
* putop: Set the value of a parameter that controls the operation
*        of the optimization subroutines.
*
* Update history
*    v1.0  18-Nov-1992  Original version
*    v1.1  20-Jul-1998  Small changes
*    v1.2   5-Aug-1999  Small format change
*    v1.3   6-Aug-1999  Correct error-reporting bug
*
      implicit none
*
* Arguments
*    cvar  = c*(?) = name of routine.variable
*    value = r*4   = value to set (-ve => use default)
*
      character*(*) cvar
      real*4        value
*
* Common block
*    / OPHOLD / holds values of all control variables
*        CRVAR = c*16(256) = name of variable i
*        VLMIN = r*4(256)  = minimum value of variable i
*        VLDEF = r*4(256)  = default value of variable i
*        VLMAX = r*4(256)  = maximum value of variable i
*        VLSET = r*4(256)  = set value of variable i
*        IINIT = i*4       = flag set if initialization done
*
      common /OPHOLD/ CRVAR,VLMIN,VLDEF,VLMAX,VLSET,IINIT
      character*16    CRVAR(256)
      real*4          VLMIN(256),VLDEF(256),VLMAX(256),VLSET(256)
      integer*4       IINIT
*
* Local variables
*    i1 = i*4 = counting index
*    i2 = i*4 = set if a match has been found
*
      integer*4 i1,i2
*
* If initialization not done, return with error message
*
      if (IINIT .le. 0) then
        write (6,699)
        return
      endif
*
* Set match variable
*
      i2=0
*
* Run through list of variable names looking for a match
*
      do i1=1,256
        if (cvar .eq. CRVAR(i1)) then
          if     (value .lt.       0.0) then
            VLSET(i1)=VLDEF(i1)
            i2       =1
          elseif (value .lt. VLMIN(i1)) then
            VLSET(i1)=VLMIN(i1)
            i2       =1
            write (6,696) cvar
          elseif (value .gt. VLMAX(i1)) then
            VLSET(i1)=VLMAX(i1)
            i2       =1
            write (6,697) cvar
          else
            VLSET(i1)=value
            i2       =1
          endif
        endif
      enddo
*
* If no match, report problem
*
      if (i2 .le. 0) then
        write (6,698) cvar
      endif
*
* End
*
 696  format (' * Warning: putop: variable ',a,' hit lower limit *')
 697  format (' * Warning: putop: variable ',a,' hit upper limit *')
 698  format (' *** Error: putop: variable ',a,' unknown       ***')
 699  format (' *** Error: putop: initialise using iniop first ***')
*
      end
