c
      real*4 function ranuni(idum)
c
c ranuni: Uses Minimal Standard random number generator with 
c         shuffling to get a long-sequence random deviate in
c         0.0-1.0. From Press et al. May be used ONLY with
c                  a = 16807, q = 127773, r =  2836
c                  a = 48271, q =  44488, r =  3399
c                  a = 69621, q =  30845, r = 23902
c
      implicit none
c
c Argument
c    idum = i*4 = seed value
c
      integer*4 idum
c
c Parameters
c    EPS  = r*4 = allowed minimum value
c    IA   = i*4 = a-value in Schrage's algorithm
c    IQ   = i*4 = q-value in Schrage's algorithm
c    IR   = i*4 = r-value in Schrage's algorithm
c    NTAB = i*4 = length of shuffle table
c
      real*4    EPS
      integer*4 IA,IQ,IR,NTAB
      parameter (EPS =     1.2e-07)
      parameter (IA  = 16807)
      parameter (IQ  =127773)
      parameter (IR  =  2836)
      parameter (NTAB=    32)
c
c Variables
c    am   = r*4       = inverse of modulus for shuffle
c    i1   = i*4       = counting index in shuffle table
c    i2   = i*4       = intermediate variable in modulus calc
c    im   = i*4       = modulus for shuffle (aq+r)
c    iv   = i*4(NTAB) = shuffle table
c    iy   = i*4       = random deviate
c    ndiv = i*4       = divisor for table indexing
c    rnmx = r*4       = maximum random number .neq. 1
c
      real*4    am,rnmx
      integer*4 iv(NTAB)
      integer*4 i1,i2,im,iy,ndiv
c
c Initial values for table entries
c
      data iv /NTAB*0/
      data iy /     0/
      save iv,iy
c
c Get modulus for shuffle
c
      im=ia*iq+ir
c
c Get am
c
      am=1.0d0/dfloat(im)
c
c Get divisor for indexing
c
      ndiv=1+(im-1)/NTAB
c
c Get maximum random deviate allowed
c
      rnmx=1.0-EPS
c
c Initialise, checking that idum is not zero
c
      if ((idum .le. 0) .or. (iy .eq. 0)) then
c
c Make idum better
c
        idum=max(-idum,1)
c
c Load shuffle table, discarding first 8 values
c
        do i1=NTAB+8,1,-1
c
c Get next random number using Schrage's method
c
          i2  =idum/IQ
          idum=IA*(idum-i2*IQ)-(IR*i2)
          if (idum .lt. 0) then
            idum=idum+im
          endif
c
c Store in shuffle table if appropriate
c
          if (i1 .le. NTAB) then
            iv(i1)=idum
          endif
c
        enddo
        iy=iv(1)
c
      endif
c
c Initialising already complete: calculate idum=mod(IA*idum,im)
c    without overflows using Schrage's method
c
      i2  =idum/IQ
      idum=IA*(idum-i2*IQ)-(IR*i2)
      if (idum .lt. 0) then
        idum=idum+im
      endif
c
c Get number in range 1 - NTAB
c
      i1=1+(iy/ndiv)
c
c Output previously stored value, replenish shuffle table
c
      iy    =iv(i1)
      iv(i1)=idum
c
c Uniform random deviate in range (small) - (just less than 1)
c
      ranuni=min(am*float(iy),rnmx)
c
c End
c
      end
