c
      subroutine sampl(func,xvlo,xvec,xvhi,nvec,fval,nval)
c
c sampl: Take many Monte-Carlo samples of func, returning an array
c        of values that can be used to examine the distribution.
c
c Update history
c    V1.0   8-Oct-1992  Original version
c    V2.0  20-Jul-1998  Remove plotting
c
      implicit none
c
c Arguments
c    func = r*4 function = quantity to minimize
c    xvlo = r*4(nvec)    = array of lower limits to xvec
c    xvec = r*4(nvec)    = array of quantities to find at minimum
c    xvhi = r*4(nvec)    = array of upper limits to xvec
c    nvec = i*4          = number of parameters
c    fval = r*4(nval)    = values calculated
c    nval = i*4          = length of fval array
c
      integer*4 nvec,nval
      real*4    func
      external  func
      real*4    xvlo(nvec),xvec(nvec),xvhi(nvec),fval(nval)
c
c Subprogrammes
c    ranuni = r*4 function = uniform random number generator
c
      real*4   ranuni
      external ranuni
c
c Local parameter
c    NVMAX = i*4 = maximum size of vectors that can be dealt with
c
      integer*4  NVMAX
      parameter (NVMAX=128)
c
c Local variables
c    i1    = i*4        = counting index
c    i2    = i*4        = counting index
c    iseed = i*4        = random number generator flag
c    value = r*4        = value from function call
c    xscr  = r*4(NVMAX) = scratch vector argument
c
      real*4    xscr(NVMAX)
      real*4    value
      integer*4 i1,i2,iseed
c
c Get random number generator
c
      call getop('RANUNI.ISEED',value)
      iseed=ifix(value+0.5)
c
c Check that vector length isn't too large
c
      if (nvec .gt. NVMAX) then
        write (6,699)
        return
      endif
c
c Copy initial vector value to scratch vector
c
      do i2=1,nvec
        xscr(i2)=xvec(i2)
      enddo
c
c Get starting value of func
c
      fval(1)=func(xscr,nvec)
c
c Now, get remaining (nval-1) values of func, randomly
c    sampled from the xvlo - xvhi range
c
      do i1=2,nval
c
c For each variable, calculate the corresponding value using a 
c    uniform distribution in xvlo to xvhi.
c
        do i2=1,nvec
          xscr(i2)=xvlo(i2)+(xvhi(i2)-xvlo(i2))*ranuni(iseed)
        enddo
c
c Store new value of function
c
        fval(i1)=func(xscr,nvec)
c
      enddo
c
c Save random number generator seed
c
      value=float(iseed)
      call putop('RANUNI.ISEED',value)
c
c End
c
 699  format (' *** Error: sampl: parameter vector too large ***')
c
      end
