*
      program update_file
*
* update_file: Make single evaluation and minimization on the configuration 
*              picked up from an input file
*
* Update history:
*    v1.0  19-May-2004  Original version
*    v1.1  25-May-2004  Correct some minor issues
*    v1.2  27-May-2004  Add a final smplx loop, rename
*    v1.3  28-May-2004  Remove smplx loop
*
      implicit none
*
* Include file
*
      include 'eval.inc'
*
* Subprogrammes
*    econf = r*4 function = energy of configuration per charge
*    error = subroutine   = info about gradients in min
*    iargc = i*4 function = number of arguments
*    iniop = subroutine   = set up minimization code variables
*    optim = subroutine   = minimization code under test
*    sampl = subroutine   = get function sample values
*
      real*4    econf
      integer*4 iargc
      external  econf,error,iniop,optim,sampl
*
* Local parameters
*    CNAME  = c*11 = code name
*    CUPDTE = c*11 = code date
*    CVERS  = c*4  = code version
*
      character*11 CUPDTE
      character*11 CNAME
      character*4  CVERS
      parameter   (CNAME='update_file')
      parameter   (CVERS='v1.2')
      parameter   (CUPDTE='27-May-2004')
*
* Local variables
*    buff   = c*132       = input buffer
*    cfilin = c*132       = input file name
*    cfilou = c*132       = output file name
*    ctitle = c*132       = title string
*    edelt  = r*4         = change in minimum energy
*    emin   = r*4         = minimum found
*    fbar   = r*4         = average of samples
*    fmax   = r*4         = maximum of samples
*    fmin   = r*4         = minimum of samples
*    fsd    = r*4         = s.d. of samples
*    fval   = r*4(NSAMPL) = samples of function
*    i1     = i*4         = counting variable
*    i2     = i*4         = counting variable
*    i3     = i*4         = counting variable
*    iopt   = i*4         = type of minimization to perform
*    nc     = i*4         = number of particles
*    ntitle = i*4         = length of title string
*    nvec   = i*4         = length of vectors in minimization
*    s1     = r*8         = accumulator
*    xehi   = r*4(NXMAX)  = parameter vector high error
*    xelo   = r*4(NXMAX)  = parameter vector low error
*    xvec   = r*4(NXMAX)  = parameter vector in minimization
*    xvlo   = r*4(NXMAX)  = parameter vector low bound
*    xvhi   = r*4(NXMAX)  = parameter vector high bound
*
      character*132 buff,cfilin,cfilou,ctitle
      real*8        s1
      real*4        edelt,emin,fbar,fmax,fmin,fsd,x1,x2,x3,x4,x5,x6
      real*4        xehi(NXMAX),xelo(NXMAX),xvec(NXMAX),
     +              xvhi(NXMAX),xvlo(NXMAX)
      real*4        fval(NSAMPL)
      integer*4     i1,i2,i3,ierror,nc,nfile,nloop,ntitle,nvec
      integer*4     iopt(NLMAX)
*
* Compose and write title
*
      call strhed(CNAME,CVERS,CUPDTE,ctitle)
      call strlen(ctitle,ntitle)
      write (6,600) ctitle(1:ntitle)
*
* Get input and output file names
*
      if (iargc() .eq. 2) then
        call getarg(1,cfilin)
        call strlen(cfilin,nfile)
        write (6,601) cfilin(1:nfile)
*
        call getarg(2,cfilou)
        call strlen(cfilou,nfile)
        write (6,602) cfilou(1:nfile)
      else
        write (6,697)
        stop ' '
      endif
*
* Read from input file
*
      open(unit=4,name=cfilin,status='old',iostat=ierror)
      if (ierror .ne. 0) then
        write (6,695)
        stop ' '
      endif
*
      dowhile (ierror .eq. 0) 
        read (4,400,iostat=ierror) buff
        if (ierror .eq. 0) then
          if (buff(1:1) .ne. '#') then
*
* Not a comment line: read charge index, angle data
*
            read (buff,*) nc,x1,x2,x3,x4,x5,x6
*
* Store data
*
            i1      =2*nc-1
            xvlo(i1)=x1
            xvec(i1)=x2
            xvhi(i1)=x3
            i1      =2*nc
            xvlo(i1)=x4
            xvec(i1)=x5
            xvhi(i1)=x6
*
          endif
        endif
      enddo
*
* Close file
*
      close (unit=4)
      write (6,603) nc 
*
* Initialise optimization variables
*
      call iniop
      call putop('POWLL.EPS',1.0e-07)
*
* Set basic optimization method: simple monpo followed by
* smplx
*
      iopt(1)=9
      iopt(2)=4
*
* Get number of parameters. Since I will not force particle 1 to
* be on the z-axis explicitly (it is done implicitly using
* the bounds array), this is twice the number of particles
*
      nvec=2*nc
*
* Check not too many particles
*
      if     (nvec .gt. NXMAX) then
        write (6,699) 
        stop ' '
      elseif (nvec .lt.     4) then
        write (6,698)
        stop ' '
      endif
*
* Sample space of function, and find rough statistics of results
*
      call sampl(econf,xvlo,xvec,xvhi,nvec,fval,NSAMPL)
*
* Find minimum, maximum, mean, standard deviation of sampled values.
*
      s1=0.0d0
      do i1=1,NSAMPL
        s1=s1+dble(fval(i1))
      enddo
      fbar=sngl(s1/dble(float(NSAMPL)))
*
      s1=0.0d0
      do i1=1,NSAMPL
        s1=s1+dble((fval(i1)-fbar)**2)
      enddo
      fsd=sngl(dsqrt(s1/dble(float(NSAMPL-1))))
* 
      fmin=fval(1)
      fmax=fmin
      do i1=1,NSAMPL
        fmin=min(fmin,fval(i1))
        fmax=max(fmax,fval(i1))
      enddo
*
      emin=econf(xvec,nvec)
*
* Print starting conditions 
*
      write (6,604) fmin,fmax,fbar,fsd,emin
*
* Call optimization routine with different options
*
      nloop=1
      call optim(econf,xvlo,xvec,xvhi,nvec,iopt(nloop))
      emin =econf(xvec,nvec)
*
      nloop=2
      call optim(econf,xvlo,xvec,xvhi,nvec,iopt(nloop))
*
* Get variation in solution for small change in energy
*
      emin =econf(xvec,nvec)
      edelt=0.001*emin
      call error(econf,edelt,xvlo,xelo,xvec,xehi,xvhi,nvec)
*
* Print result
*
      write (6,605) edelt
      do i1=1,nvec
        x1=0.5*abs(xehi(i1)-xelo(i1))
        write (6,606) i1,xvlo(i1),xvec(i1),x1,xvhi(i1)
      enddo
      write (6,607) emin
*
* Write results to output
*
      open(unit=7,name=cfilou,status='unknown',iostat=ierror)
      if (ierror .ne. 0) then
        write (6,696)
        stop ' '
      endif
*
      call strlen(cfilou,nfile)
      write (7,700) cfilou(1:nfile)
      write (7,701) emin
      do i1=1,nc
        i2=2*i1-1
        i3=2*i1
        write (7,702) i1,xvlo(i2),xvec(i2),xvhi(i2),
     +                   xvlo(i3),xvec(i3),xvhi(i3)
      enddo
      write (7,703) nc
*
* Close file
*
      close (unit=7)
*
* End of run
*
      write (6,608) ctitle(1:ntitle)
*
 100  format (i2,1x,a)
 101  format (' ',i2,' ''',a,'''')
*
 400  format (a)
 401  format (' ',a)
*
 600  format (' '/
     +        ' ',a//
     +        '    Evaluate energy function and do quick',
     +        ' monpo minimization.'/)
 601  format ('            Input file = ',a)
 602  format ('           Output file = ',a)
 603  format ('     Number of charges = ',i2)
 604  format (' '/
     +        '     Rough function range ',1pe15.5,' to ',1pe15.5/
     +        '                     mean ',1pe15.5,' sd ',1pe15.5/
     +        '         initial E/charge ',1pe15.5)
 605  format (' '/
     +        ' Table of results and change for ',f6.3,
     +        ' energy increase'/
     +        ' index',5X,'minimum',10x,'end (change)',
     +        10x,'maximum')
 606  format (' ',i5,2f12.3,' (',f8.3,') ',f12.3)
 607  format ('           final E/charge ',1pe15.5)
 608  format (' '/
     +        ' ',a)
*
 695  format (' *** Error:  input file access error ***')
 696  format (' *** Error: output file access error ***')
 697  format (' '/
     +        ' Usage: eval infile outfile'/)
 698  format (' *** Error: too few particles for interest ***')
 699  format (' *** Error: too many particles for arrays  ***')
*
 700  format ('#'/
     +        '# ',a/
     +        '#')
 701  format ('# emin = ',1pe15.5/
     +        '#')
 702  format (i3,6f10.4)
 703  format ('#'/
     +        '#',i3,' charges'/
     +        '#')
*
      end
